package com.picfun.bt_pio;

import android.os.Bundle;
import android.app.Activity;
import android.os.Handler;
import android.os.Message;
import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.graphics.Color;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import java.util.Timer;
import java.util.TimerTask;

@SuppressLint({ "HandlerLeak", "DrawAllocation" })
public class BT_PIO extends Activity {
	/**** Bluetooth関連クラス定数定義 ****/
	public static final int CONNECTDEVICE = 1;
	public static final int ENABLEBLUETOOTH = 2;
	private BluetoothAdapter BTadapter;
	private BluetoothClient BTclient;
	private byte[] RcvPacket = new byte[32];
	private byte[] SndPacket = new byte[32];	
	/**** GUI関連クラス定数定義 **********/
	private Button Connect, Cont1, Cont2;
	private TextView State, State1, State2, Mesr1, Mesr2;
	/**** バッファ、一般変数定義 *********/	
	static Timer timer;
	static int ContFlag1 = 0;
	static int ContFlag2 = 0;
	private static int Data1, Data2;
	
	/******** 最初に実行するメソッド *********************/
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);	
		setContentView(R.layout.main);					// 画面表示
        //コンポーネントの設定
        Connect = (Button)this.findViewById(R.id.button1);
        Cont1 = (Button)this.findViewById(R.id.button2);
        Cont2 = (Button)this.findViewById(R.id.button3);
        State = (TextView)this.findViewById(R.id.textView1);
        State1 = (TextView)this.findViewById(R.id.textView2);
        State2 = (TextView)this.findViewById(R.id.textView3);       
        Mesr1 = (TextView)this.findViewById(R.id.textView4);
        Mesr2 = (TextView)this.findViewById(R.id.textView5);             
        // ボタンイベント組み込み
        Connect.setOnClickListener((OnClickListener) new connect());		
        Cont1.setOnClickListener((OnClickListener) new control1());
        Cont2.setOnClickListener((OnClickListener)new control2());		
		// Bluetoothが有効な端末か確認
		BTadapter = BluetoothAdapter.getDefaultAdapter();
		if (BTadapter == null) {
			State.setTextColor(Color.YELLOW);
			State.setText("Bluetooth未サポート");
		}		
	}
	/************* アクティビティ開始時 ******************/
	@Override
	public void onStart() {
		super.onStart();
		if (BTadapter.isEnabled() == false) {		// Bluetoorhが有効でない場合
			// Bluetoothを有効にするダイアログ画面に遷移し有効化要求
			Intent BTenable = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			// 有効化されて戻ったらENABLEパラメータ発行
			startActivityForResult(BTenable, ENABLEBLUETOOTH);
		}
		else {
			if (BTclient == null) {
				// BluetoothClientをハンドラで生成
				BTclient = new BluetoothClient(this, handler);
			}
		}
	}
	/********* アクティビティ再開時 **********************/
	@Override
	public synchronized void onResume() {
		super.onResume();
		// 特に処理なし
	}
	/******** アクティビティ破棄時 ***********************/
	@Override
	public void onDestroy() {
		super.onDestroy();
		if(timer != null)			// timerが起動中なら終了
			timer.cancel();
		if (BTclient != null)		// Bluetoothを使用中なら終了
			BTclient.stop();
	}
	/******** 遷移ダイアログからの戻り処理 ***************/
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
			// 端末選択ダイアログからの戻り処理
			case CONNECTDEVICE: 							// 端末が選択された場合
				if (resultCode == Activity.RESULT_OK) {
					String address = data.getExtras().getString(DeviceListActivity.DEVICEADDRESS);
					// 生成した端末に接続要求
					BluetoothDevice device = BTadapter.getRemoteDevice(address);
					BTclient.connect(device);				// 端末へ接続
				}
				break;
			// 有効化ダイアログからの戻り処理
			case ENABLEBLUETOOTH: 							// Bluetooth有効化の場合
				if (resultCode == Activity.RESULT_OK) {
					// 正常に有効化できたらクライアントクラスを生成しハンドラを生成する
					BTclient = new BluetoothClient(this, handler);
				}
				else {
					Toast.makeText(this, "Bluetoothのサポートなし", Toast.LENGTH_SHORT).show();
					finish();
				}
		}
	}
    /********* 接続ボタンイベントクラス ******************/
    class connect implements OnClickListener{
    	public void onClick(View v){
 			// デバイス検索と選択ダイアログへ移行    		
    		Intent Intent = new Intent(BT_PIO.this, DeviceListActivity.class);
 			// 端末が選択されて戻ったら接続要求するようにする
 			startActivityForResult(Intent, CONNECTDEVICE);   	    		
    	}
    }
	/******* BT端末接続処理のハンドラ ********************/
	private final Handler handler = new Handler() {
		// ハンドルメッセージごとの処理
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
				// 接続途中の場合
				case BluetoothClient.MESSAGE_STATECHANGE:
					switch (msg.arg1) {
						case BluetoothClient.STATE_CONNECTED:
							State.setTextColor(Color.BLACK);
							State.setText("接続完了");
			   		    	//** タイマ起動と周期起動スレッドのスタート
			   		    	timer = new Timer(true);		// Daeeamonとして生成
			   		    	timer.schedule(new TimerTask(){
				    			@Override
				    			public void run(){
				    				//** 周期的に状態要求を実行 
				    	      		SndPacket[0] = 0x53;				// S
				    				SndPacket[1] = 0x42;				// B
				    				SndPacket[2] = 0x45;				// E
				    				BTclient.write(SndPacket); 			// 送信実行			    				
				    			}
			   				}, 1000, 500);			// 最初1秒待ち、以後0.5秒周期
							break;
						case BluetoothClient.STATE_CONNECTING:
							State.setTextColor(Color.BLACK);
							State.setText("接続中");
							break;
						case BluetoothClient.STATE_NONE:
							State.setTextColor(Color.RED);
							State.setText("接続失敗");
							break;
					}
					break;
			// BT受信処理の場合
			case BluetoothClient.MESSAGE_READ:
				State.setText("送受信処理中");
				RcvPacket = (byte[])msg.obj;
				// 受信処理実行
				Process();
				break;
			}
		}
	}; 
    /**** 制御１ボタンイベント処理サブクラス *************/
    class control1 implements OnClickListener{
    	public void onClick(View v){
    		if(Cont1.getText() == "Off")
    			ContFlag1 = 1;
    		else
    			ContFlag1 = 2;
    	}
    }
    /**** 制御2ボタンイベント処理サブクラス **************/
    class control2 implements OnClickListener{
    	public void onClick(View v){
    		if(Cont2.getText() == "Off")
    			ContFlag2 = 1;
    		else
    			ContFlag2 = 2;
    	}  
    } 
    /********** 受信データ処理メソッド *******************/
    private void Process(){
		if((RcvPacket[0] == 0x4D)&&(RcvPacket[1] == 0x42)){		// MB
			if(RcvPacket[2] == 0x31){
				Cont1.setBackgroundColor(Color.RED);			// 制御1
				Cont1.setText("On");
			}
			else{
				Cont1.setBackgroundColor(Color.GREEN);			// 制御1
				Cont1.setText("Off");
			}
			if(RcvPacket[3] == 0x31){
				Cont2.setBackgroundColor(Color.RED);			// 制御2
				Cont2.setText("On");
			}
			else{
				Cont2.setBackgroundColor(Color.GREEN);			// 制御2
				Cont2.setText("Off");
			}
			/*** 状態データ処理****/
			if(RcvPacket[4] == 0x30){
				State1.setBackgroundColor(Color.RED);			// 状態1
				State1.setText("On");
			}
			else{
				State1.setBackgroundColor(Color.GREEN);			// 状態1
				State1.setText("Off");
			}
			if(RcvPacket[5] == 0x30){
				State2.setBackgroundColor(Color.RED);			// 状態2
				State2.setText("On");
			}
			else{
				State2.setBackgroundColor(Color.GREEN);			// 状態2
				State2.setText("Off");
			}
			/*** 計測データ処理 ***/
			Data1 = (RcvPacket[6]-0x40)*32 + RcvPacket[7]-0x40;
			Mesr1.setText(Integer.toString(Data1));				// 計測1
			Data2 = (RcvPacket[8]-0x40)*32 + RcvPacket[9]-0x40;	   								
			Mesr2.setText(Integer.toString(Data2));	   			// 計測2			
		}
		/** 制御1実行 **/
		if(ContFlag1 == 1){
      		SndPacket[0] = 0x53;				// S
			SndPacket[1] = 0x43;				// C
			SndPacket[2] = 0x31;				// 1
			SndPacket[3] = 0x31;				// 1
			SndPacket[4] = 0x45;				// E
			BTclient.write(SndPacket); 			// 送信実行				
		}
		else if(ContFlag1 == 2){
      		SndPacket[0] = 0x53;				// S
			SndPacket[1] = 0x43;				// C
			SndPacket[2] = 0x31;				// 1
			SndPacket[3] = 0x30;				// 0
			SndPacket[4] = 0x45;				// E
			BTclient.write(SndPacket); 			// 送信実行				
		}
		ContFlag1 = 0;
		/** 制御2実行 ***/
		if(ContFlag2 == 1){
      		SndPacket[0] = 0x53;				// S
			SndPacket[1] = 0x43;				// C
			SndPacket[2] = 0x32;				// 2
			SndPacket[3] = 0x31;				// 1
			SndPacket[4] = 0x45;				// E
			BTclient.write(SndPacket); 			// 送信実行				
		}
		else if(ContFlag2 == 2){
      		SndPacket[0] = 0x53;				// S
			SndPacket[1] = 0x43;				// C
			SndPacket[2] = 0x32;				// 2
			SndPacket[3] = 0x30;				// 0
			SndPacket[4] = 0x45;				// E
			BTclient.write(SndPacket); 			// 送信実行				
		}
		ContFlag2 = 0;			
    }
}

